==> eDAL-MetaDataAPI-2.3.8-sources.jar: de/ipk_gatersleben/bit/bi/edal/sample/login/GoogleLoginModule.java <==
/*
 * Copyright (c) 2015 Leibniz Institute of Plant Genetics and Crop Plant Research (IPK), Gatersleben, Germany.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Creative Commons Attribution-NoDerivatives 4.0 International (CC BY-ND 4.0)
 * which accompanies this distribution, and is available at http://creativecommons.org/licenses/by-nd/4.0/
 *
 * Contributors:
 *      Leibniz Institute of Plant Genetics and Crop Plant Research (IPK), Gatersleben, Germany - initial API and implementation
 */
package de.ipk_gatersleben.bit.bi.edal.sample.login;

import java.io.IOException;
import java.util.Map;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.ConfirmationCallback;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.TextInputCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.FailedLoginException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;

public class GoogleLoginModule implements LoginModule {

	/** initial state */
	private Subject subject;
	private CallbackHandler callbackHandler;
	private Map<String, ?> sharedState;
	private Map<String, ?> options;

	/** the authentication status */
	private boolean succeeded = false;
	private boolean commitSucceeded = false;

	/** username and password */
	private String username;
	private char[] password;
	private boolean keepLogin;

	/** GooglePrincipal */
	private GooglePrincipal principal;

	/**
	 * Initialize this <code>LoginModule</code>.
	 * 
	 * @param subject
	 *            the <code>Subject</code> to be authenticated.
	 * @param callbackHandler
	 *            a <code>CallbackHandler</code> for communicating with the end
	 *            user (prompting for user names and passwords, for example).
	 * @param sharedState
	 *            shared <code>LoginModule</code> state.
	 * @param options
	 *            options specified in the login <code>Configuration</code> for
	 *            this particular <code>LoginModule</code>.
	 */
	public void initialize(Subject subject, CallbackHandler callbackHandler, Map<String, ?> sharedState, Map<String, ?> options) {

		this.subject = subject;
		this.callbackHandler = callbackHandler;
		this.setSharedState(sharedState);
		this.setOptions(options);
		this.keepLogin = false;

	}

	/**
	 * Authenticate the user by prompting for a user name and password.
	 * 
	 * @return true in all cases since this <code>LoginModule</code> should not
	 *         be ignored.
	 * @exception FailedLoginException
	 *                if the authentication fails.
	 * @exception LoginException
	 *                if this <code>LoginModule</code> is unable to perform the
	 *                authentication.
	 */
	public boolean login() throws LoginException {

		/** prompt for a user name and password */
		if (callbackHandler == null) {
			throw new LoginException("Error: no CallbackHandler available " + "to get authentication information from the user");
		}

		Callback[] callbacks = new Callback[4];
		try {
			callbacks[0] = new NameCallback(" ");
			callbacks[1] = new PasswordCallback(" ", false);
			callbacks[2] = new ConfirmationCallback(ConfirmationCallback.INFORMATION, ConfirmationCallback.YES_NO_OPTION, ConfirmationCallback.NO);
			callbacks[3] = new TextInputCallback("proxy");
		} catch (Exception e) {
			e.printStackTrace();
		}
		try {
			callbackHandler.handle(callbacks);
			username = ((NameCallback) callbacks[0]).getName();

			char[] tmpPassword = ((PasswordCallback) callbacks[1]).getPassword();
			if (tmpPassword == null) {
				/** treat a NULL password as an empty password */
				tmpPassword = new char[0];
			}
			password = new char[tmpPassword.length];
			System.arraycopy(tmpPassword, 0, password, 0, tmpPassword.length);
			((PasswordCallback) callbacks[1]).clearPassword();

			if (((ConfirmationCallback) callbacks[2]).getSelectedIndex() == ConfirmationCallback.YES) {
				keepLogin = true;
			}

			if (username == null && password.length == 0) {
				throw new LoginException("Authentication stopped by User");
			}

		} catch (IOException ioe) {
			throw new LoginException(ioe.toString());
		} catch (UnsupportedCallbackException uce) {
			throw new LoginException("Error: " + uce.getCallback().toString() + " not available to garner authentication information " + "from the user");
		}
		/*
		 * Enter here the code for verify the user name and password
		 */

		try {
			GoogleOauth.authenticateGoogleUser(username, new String(password), keepLogin, ((TextInputCallback) callbacks[3]).getText());
		} catch (Exception e) {
			throw new LoginException(e.getMessage());
		}

		succeeded = true;
		return true;
	}

	/**
	 * This method is called if the LoginContext's overall authentication
	 * succeeded (the relevant REQUIRED, REQUISITE, SUFFICIENT and OPTIONAL
	 * LoginModules succeeded).
	 * 
	 * If this LoginModule's own authentication attempt succeeded (checked by
	 * retrieving the private state saved by the <code>login</code> method),
	 * then this method associates a <code>GooglePrincipal</code> with the
	 * <code>Subject</code> located in the <code>LoginModule</code>. If this
	 * LoginModule's own authentication attempted failed, then this method
	 * removes any state that was originally saved.
	 * 
	 * @exception LoginException
	 *                if the commit fails.
	 * 
	 * @return true if this LoginModule's own login and commit attempts
	 *         succeeded, or false otherwise.
	 */
	public boolean commit() throws LoginException {
		if (!succeeded) {
			return false;
		} else {

			/** add a principal (authenticated identity) to the Subject */
			principal = new GooglePrincipal(username);

			if (!subject.getPrincipals().contains(principal)) {
				subject.getPrincipals().add(principal);
			}

			/** in any case, clean out state */
			username = null;
			for (int i = 0; i < password.length; i++) {
				password[i] = ' ';
			}
			password = null;

			commitSucceeded = true;
			return true;
		}
	}

	/**
	 * This method is called if the LoginContext's overall authentication
	 * failed. (the relevant REQUIRED, REQUISITE, SUFFICIENT and OPTIONAL
	 * LoginModules did not succeed).
	 * 
	 * If this LoginModule's own authentication attempt succeeded (checked by
	 * retrieving the private state saved by the <code>login</code> and
	 * <code>commit</code> methods), then this method cleans up any state that
	 * was originally saved.
	 * 
	 * @exception LoginException
	 *                if the abort fails.
	 * @return false if this LoginModule's own login and/or commit attempts
	 *         failed, and true otherwise.
	 */
	public boolean abort() throws LoginException {
		if (!succeeded) {
			return false;
		} else if (succeeded && !commitSucceeded) {
			/** login succeeded but overall authentication failed */
			succeeded = false;
			username = null;
			if (password != null) {
				for (int i = 0; i < password.length; i++) {
					password[i] = ' ';
				}
				password = null;
			}
			principal = null;
		} else {
			/**
			 * overall authentication succeeded and commit succeeded, but
			 * someone else's commit failed
			 */
			logout();
		}
		return true;
	}

	/**
	 * Logout the user.
	 * 
	 * This method removes the <code>OAuthPrincipal</code> that was added by the
	 * <code>commit</code> method.
	 * 
	 * @exception LoginException
	 *                if the logout fails.
	 * @return true in all cases since this <code>LoginModule</code> should not
	 *         be ignored.
	 */
	public boolean logout() throws LoginException {

		subject.getPrincipals().remove(principal);
		succeeded = commitSucceeded;
		username = null;
		if (password != null) {
			for (int i = 0; i < password.length; i++) {
				password[i] = ' ';
			}
			password = null;
		}
		principal = null;
		return true;
	}

	/**
	 * @return the sharedState
	 */
	public Map<String, ?> getSharedState() {
		return sharedState;
	}

	/**
	 * @param sharedState
	 *            the sharedState to set
	 */
	public void setSharedState(Map<String, ?> sharedState) {
		this.sharedState = sharedState;
	}

	/**
	 * @return the options
	 */
	public Map<String, ?> getOptions() {
		return options;
	}

	/**
	 * @param options
	 *            the options to set
	 */
	public void setOptions(Map<String, ?> options) {
		this.options = options;
	}
}

==> eDAL-MetaDataAPI-2.4.3-sources.jar: de/ipk_gatersleben/bit/bi/edal/sample/login/GoogleLoginModule.java <==
/*
 * Copyright (c) 2015 Leibniz Institute of Plant Genetics and Crop Plant Research (IPK), Gatersleben, Germany.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Creative Commons Attribution-NoDerivatives 4.0 International (CC BY-ND 4.0)
 * which accompanies this distribution, and is available at http://creativecommons.org/licenses/by-nd/4.0/
 *
 * Contributors:
 *      Leibniz Institute of Plant Genetics and Crop Plant Research (IPK), Gatersleben, Germany - initial API and implementation
 */
package de.ipk_gatersleben.bit.bi.edal.sample.login;

import java.io.IOException;
import java.util.Map;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.ConfirmationCallback;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.callback.TextInputCallback;
import javax.security.auth.callback.UnsupportedCallbackException;
import javax.security.auth.login.FailedLoginException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;

@Deprecated
public class GoogleLoginModule implements LoginModule {

	/** initial state */
	private Subject subject;
	private CallbackHandler callbackHandler;
	private Map<String, ?> sharedState;
	private Map<String, ?> options;

	/** the authentication status */
	private boolean succeeded = false;
	private boolean commitSucceeded = false;

	/** username and password */
	private String username;
	private char[] password;
	private boolean keepLogin;

	/** GooglePrincipal */
	private GooglePrincipal principal;

	/**
	 * Initialize this <code>LoginModule</code>.
	 * 
	 * @param subject
	 *            the <code>Subject</code> to be authenticated.
	 * @param callbackHandler
	 *            a <code>CallbackHandler</code> for communicating with the end
	 *            user (prompting for user names and passwords, for example).
	 * @param sharedState
	 *            shared <code>LoginModule</code> state.
	 * @param options
	 *            options specified in the login <code>Configuration</code> for
	 *            this particular <code>LoginModule</code>.
	 */
	public void initialize(Subject subject, CallbackHandler callbackHandler, Map<String, ?> sharedState,
			Map<String, ?> options) {

		this.subject = subject;
		this.callbackHandler = callbackHandler;
		this.setSharedState(sharedState);
		this.setOptions(options);
		this.keepLogin = false;

	}

	/**
	 * Authenticate the user by prompting for a user name and password.
	 * 
	 * @return true in all cases since this <code>LoginModule</code> should not
	 *         be ignored.
	 * @exception FailedLoginException
	 *                if the authentication fails.
	 * @exception LoginException
	 *                if this <code>LoginModule</code> is unable to perform the
	 *                authentication.
	 */
	public boolean login() throws LoginException {

		/** prompt for a user name and password */
		if (callbackHandler == null) {
			throw new LoginException(
					"Error: no CallbackHandler available " + "to get authentication information from the user");
		}

		Callback[] callbacks = new Callback[4];
		try {
			callbacks[0] = new NameCallback(" ");
			callbacks[1] = new PasswordCallback(" ", false);
			callbacks[2] = new ConfirmationCallback(ConfirmationCallback.INFORMATION,
					ConfirmationCallback.YES_NO_OPTION, ConfirmationCallback.NO);
			callbacks[3] = new TextInputCallback("proxy");
		} catch (Exception e) {
			e.printStackTrace();
		}
		try {
			callbackHandler.handle(callbacks);
			username = ((NameCallback) callbacks[0]).getName();

			char[] tmpPassword = ((PasswordCallback) callbacks[1]).getPassword();
			if (tmpPassword == null) {
				/** treat a NULL password as an empty password */
				tmpPassword = new char[0];
			}
			password = new char[tmpPassword.length];
			System.arraycopy(tmpPassword, 0, password, 0, tmpPassword.length);
			((PasswordCallback) callbacks[1]).clearPassword();

			if (((ConfirmationCallback) callbacks[2]).getSelectedIndex() == ConfirmationCallback.YES) {
				keepLogin = true;
			}

			if (username == null && password.length == 0) {
				throw new LoginException("Authentication stopped by User");
			}

		} catch (IOException ioe) {
			throw new LoginException(ioe.toString());
		} catch (UnsupportedCallbackException uce) {
			throw new LoginException("Error: " + uce.getCallback().toString()
					+ " not available to garner authentication information " + "from the user");
		}
		/*
		 * Enter here the code for verify the user name and password
		 */

		try {
			GoogleOauth.authenticateGoogleUser(username, new String(password), keepLogin,
					((TextInputCallback) callbacks[3]).getText());
		} catch (Exception e) {
			throw new LoginException(e.getMessage());
		}

		succeeded = true;
		return true;
	}

	/**
	 * This method is called if the LoginContext's overall authentication
	 * succeeded (the relevant REQUIRED, REQUISITE, SUFFICIENT and OPTIONAL
	 * LoginModules succeeded).
	 * 
	 * If this LoginModule's own authentication attempt succeeded (checked by
	 * retrieving the private state saved by the <code>login</code> method),
	 * then this method associates a <code>GooglePrincipal</code> with the
	 * <code>Subject</code> located in the <code>LoginModule</code>. If this
	 * LoginModule's own authentication attempted failed, then this method
	 * removes any state that was originally saved.
	 * 
	 * @exception LoginException
	 *                if the commit fails.
	 * 
	 * @return true if this LoginModule's own login and commit attempts
	 *         succeeded, or false otherwise.
	 */
	public boolean commit() throws LoginException {
		if (!succeeded) {
			return false;
		} else {

			/** add a principal (authenticated identity) to the Subject */
			principal = new GooglePrincipal(username);

			if (!subject.getPrincipals().contains(principal)) {
				subject.getPrincipals().add(principal);
			}

			/** in any case, clean out state */
			username = null;
			for (int i = 0; i < password.length; i++) {
				password[i] = ' ';
			}
			password = null;

			commitSucceeded = true;
			return true;
		}
	}

	/**
	 * This method is called if the LoginContext's overall authentication
	 * failed. (the relevant REQUIRED, REQUISITE, SUFFICIENT and OPTIONAL
	 * LoginModules did not succeed).
	 * 
	 * If this LoginModule's own authentication attempt succeeded (checked by
	 * retrieving the private state saved by the <code>login</code> and
	 * <code>commit</code> methods), then this method cleans up any state that
	 * was originally saved.
	 * 
	 * @exception LoginException
	 *                if the abort fails.
	 * @return false if this LoginModule's own login and/or commit attempts
	 *         failed, and true otherwise.
	 */
	public boolean abort() throws LoginException {
		if (!succeeded) {
			return false;
		} else if (succeeded && !commitSucceeded) {
			/** login succeeded but overall authentication failed */
			succeeded = false;
			username = null;
			if (password != null) {
				for (int i = 0; i < password.length; i++) {
					password[i] = ' ';
				}
				password = null;
			}
			principal = null;
		} else {
			/**
			 * overall authentication succeeded and commit succeeded, but
			 * someone else's commit failed
			 */
			logout();
		}
		return true;
	}

	/**
	 * Logout the user.
	 * 
	 * This method removes the <code>OAuthPrincipal</code> that was added by the
	 * <code>commit</code> method.
	 * 
	 * @exception LoginException
	 *                if the logout fails.
	 * @return true in all cases since this <code>LoginModule</code> should not
	 *         be ignored.
	 */
	public boolean logout() throws LoginException {

		subject.getPrincipals().remove(principal);
		succeeded = commitSucceeded;
		username = null;
		if (password != null) {
			for (int i = 0; i < password.length; i++) {
				password[i] = ' ';
			}
			password = null;
		}
		principal = null;
		return true;
	}

	/**
	 * @return the sharedState
	 */
	public Map<String, ?> getSharedState() {
		return sharedState;
	}

	/**
	 * @param sharedState
	 *            the sharedState to set
	 */
	public void setSharedState(Map<String, ?> sharedState) {
		this.sharedState = sharedState;
	}

	/**
	 * @return the options
	 */
	public Map<String, ?> getOptions() {
		return options;
	}

	/**
	 * @param options
	 *            the options to set
	 */
	public void setOptions(Map<String, ?> options) {
		this.options = options;
	}
}
